/*
 * $Id: CloneableTester.java,v 1.5 2010/09/23 20:13:56 oboehm Exp $
 *
 * Copyright (c) 2010 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 06.08.2010 by oliver (ob@oasd.de)
 */

package patterntesting.runtime.junit;

import java.lang.reflect.*;
import java.util.*;

import org.apache.commons.logging.*;

import patterntesting.runtime.monitor.ClasspathMonitor;

/**
 * This tester checks class which implements Clonable and has therefore
 * the clone method implemented. According {@link Object#clone()} the
 * following conditions should be true:
 * <blockquote>
 * <pre>
 * x.clone() != x</pre></blockquote>
 * will be true, and that the expression:
 * <blockquote>
 * <pre>
 * x.clone().getClass() == x.getClass()</pre></blockquote>
 * will be <tt>true</tt>, but these are not absolute requirements. 
 * While it is typically the case that:
 * <blockquote>
 * <pre>
 * x.clone().equals(x)</pre></blockquote>
 * will be <tt>true</tt>, this is not an absolute requirement. 
 * <p>
 * So the equals method will be only checked if it is overwritten.
 * 
 * @author oliver
 * @since 1.0.2 (06.08.2010)
 */
public final class CloneableTester {

    private static final Log log = LogFactory.getLog(CloneableTester.class);
    private static final ClasspathMonitor classpathMonitor = ClasspathMonitor.getInstance();

    /** Utility class - no need to instantiate it. */
    private CloneableTester() {}

    /**
     * Check cloning.
     *
     * @param clazz the clazz
     */
    public static void checkCloning(Class<? extends Cloneable> clazz) {
        try {
            checkCloning(clazz.newInstance());
        } catch (InstantiationException e) {
            throw new AssertionError("can't instantiate " + clazz + ":" + e);
        } catch (IllegalAccessException e) {
            throw new AssertionError("can't access " + clazz + ":" + e);
        }
    }

    /**
     * We call the clone method of the given orig paramter.Because the
     * clone method is normally "protected" we use reflection to call it.
     * Then we compare the orig and cloned object which should be equals.
     *
     * @param orig the original object
     */
    public static void checkCloning(Cloneable orig) {
        Cloneable clone = getCloneOf(orig);
        if (ObjectTester.hasEqualsDeclared(orig.getClass())) {
            ObjectTester.checkEquals(orig, clone);
        }
    }

    /**
     * Gets the clone of the given Cloneable object.
     *
     * @param orig the orig
     * @return the clone of
     * @throws AssertionError the assertion error
     */
    public static Cloneable getCloneOf(Cloneable orig) throws AssertionError {
        try {
            Method cloneMethod = orig.getClass().getMethod("clone");
            Cloneable clone = (Cloneable) cloneMethod.invoke(orig);
            if (clone == orig) {
                throw new AssertionError(clone + " must have another reference as original object");
            }
            return clone;
        } catch (SecurityException e) {
            throw new AssertionError("can't access clone method of " + orig.getClass() + ": " + e);
        } catch (NoSuchMethodException e) {
            throw new AssertionError("no clone method found in " + orig.getClass() + ": " + e);
        } catch (IllegalAccessException e) {
            throw new AssertionError("can't access clone method of " + orig.getClass() + ": " + e);
        } catch (InvocationTargetException e) {
            throw new AssertionError("clone of " + orig.getClass() + " failed: " + e);
        }
    }
    
    /**
     * Check for each class in the given collection if it can be cloned
     * correct.
     *
     * @param classes a collection of classes to be checked
     */
    public static void checkCloning(Collection<Class<Cloneable>> classes) {
        for (Class<Cloneable> clazz : classes) {
            checkCloning(clazz);
        }
    }

    /**
     * Check for each class in the given package if it can be cloned
     * correct.
     * <br/>
     * To get a name of a package call {@link Package#getPackage(String)}.
     * But be sure that you can't get <em>null</em> as result. In this case
     * use {@link #checkCloningOfPackage(String)}.
     *
     * @param pkg the package e.g. "patterntesting.runtime"
     * @see #checkCloningOfPackage(String)
     */
    public static void checkCloning(final Package pkg) {
        assert pkg!= null;
        checkCloningOfPackage(pkg.getName());
    }

    /**
     * Check for each class in the given package if it can be cloned
     * correct.
     *
     * @param packageName the package name e.g. "patterntesting.runtime"
     */
    public static void checkCloningOfPackage(final String packageName) {
        assert packageName != null;
        Collection<Class<Cloneable>> cloneables = getCloneableClasses(packageName);
        checkCloning(cloneables);
    }

    /**
     * Check for each class in the given package if the clone method is
     * implemented correct.
     *
     * @param packageName the package name e.g. "patterntesting.runtime"
     * @param excluded classes which should be excluded from the check
     * @see #checkCloningOfPackage(String)
     */
    public static void checkCloningOfPackage(final String packageName,
            final Class<Cloneable>... excluded) {
        List<Class<Cloneable>> excludedList = Arrays.asList(excluded);
        checkCloningOfPackage(packageName, excludedList);
    }
    
    /**
     * Check for each class in the given package if the clone method is
     * implemented correct.
     *
     * @param packageName the package name e.g. "patterntesting.runtime"
     * @param excluded classes which should be excluded from the check
     * @see #checkCloningOfPackage(String)
     */
    public static void checkCloningOfPackage(final String packageName,
            final List<Class<Cloneable>> excluded) {
        Collection<Class<Cloneable>> classes = getCloneableClasses(packageName);
        log.debug(excluded + " will be excluded from check");
        ObjectTester.removeClasses(classes, excluded);
        checkCloning(classes);
    }

    @SuppressWarnings("unchecked")
    private static Collection<Class<Cloneable>> getCloneableClasses(final String packageName) {
        Collection<Class<Cloneable>> cloneables = new ArrayList<Class<Cloneable>>();
        Collection<Class<?>> concreteClasses = classpathMonitor.getConcreteClassList(packageName);
        for (Class<?> clazz : concreteClasses) {
            if (Cloneable.class.isAssignableFrom(clazz)) {
                Class<Cloneable> cloneClass = (Class<Cloneable>) clazz;
                cloneables.add(cloneClass);
                if (log.isTraceEnabled()) {
                    log.trace("Cloneable found: " + clazz);
                }
            }
        }
        return cloneables;
    }
    
}

