/*
 * (c)reated 21.07.2010 by oliver
 */
package patterntesting.runtime.junit;

import java.io.*;
import java.util.*;

import org.apache.commons.logging.*;

import patterntesting.runtime.monitor.ClasspathMonitor;
import patterntesting.runtime.util.Converter;

/**
 * This is a utility class to check the serializable nature of classes.
 * <br/>
 * NOTE: Don't use this method at the moment. Perhaps it will be part of
 * the ObjectTester class.
 * 
 * @author oliver (oliver.boehm@agentes.de)
 * @since 1.1 (21.07.2010)
 */
public final class SerializableTester {
	
	private static final Log log = LogFactory.getLog(SerializableTester.class);
    private static final ClasspathMonitor classpathMonitor = ClasspathMonitor.getInstance();

	/** Utitlity class - no need to instantiate it. */
	private SerializableTester() {}
	
    /**
     * Check serialization and deserialization of an object.
     * Because not all classes has a (correct) overwritten equals method
     * we do not compare the deserialized object with the equals() method.
     * But if it implements Comparable we verify the equality of the two
     * objects with the compareTo() method.
     *
     * @param object the object
     * @throws NotSerializableException if object can't serialized
     */
    public static void checkSerialization(final Serializable object)
            throws NotSerializableException {
    	byte[] bytes = Converter.serialize(object);
    	log.debug(object + " serialized in " + bytes.length + " bytes");
    	try {
            Object deserialized = Converter.deserialize(bytes);
            if (object instanceof Comparable<?>) {
                ObjectTester.checkCompareTo(object, deserialized);
            }
        } catch (ClassNotFoundException canthappen) {
            throw new RuntimeException(canthappen);
        }
    }
    
    /**
     * This method will create an object of the given class using the
     * default constructor. So three preconditions must be true:
     * <ol>
     *  <li>the class must not be abstract</li>
     *  <li>there must be a (public) default constructor</li>
     *  <li>it must be Serializable</li>
     * </ol>
     *
     * @param clazz the clazz
     * @throws NotSerializableException if the check fails
     */
    public static void checkSerialization(Class<Serializable> clazz) throws NotSerializableException {
        if (log.isTraceEnabled()) {
            log.trace("checking " + clazz.getName() + " if it can be serialized...");
        }
        try {
            Serializable obj = clazz.newInstance();
            checkSerialization(obj);
        } catch (InstantiationException e) {
            throw new IllegalArgumentException("can't instantiate " + clazz, e);
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException("can't access ctor of " + clazz, e);
        }
    }
    
    /**
     * Check for each class in the given collection if it can be serialized and
     * deserialized.
     *
     * @param classes a collection of classes to be checked
     * @throws NotSerializableException if one of the classes can't be serialized
     */
    public static void checkSerialization(Collection<Class<Serializable>> classes) throws NotSerializableException {
        for (Class<?> clazz : classes) {
            checkSerialization(clazz);
        }
    }

    /**
     * Check for each class in the given package if it can be serialized and
     * deserialized.
     * <br/>
     * To get a name of a package call {@link Package#getPackage(String)}.
     * But be sure that you can't get <em>null</em> as result. In this case
     * use {@link #checkSerializationOfPackage(String)}.
     *
     * @param pkg the package e.g. "patterntesting.runtime"
     * @throws NotSerializableException if one of the class can't be serialized
     * @see #checkSerializationOfPackage(String)
     */
    public static void checkSerialization(final Package pkg) throws NotSerializableException {
        assert pkg!= null;
        checkSerializationOfPackage(pkg.getName());
    }

    /**
     * Check for each class in the given package if it can be serialized and
     * deserialized.
     *
     * @param packageName the package name e.g. "patterntesting.runtime"
     * @throws NotSerializableException if one of the class can't be serialized
     */
    public static void checkSerializationOfPackage(final String packageName) throws NotSerializableException {
        assert packageName != null;
        Collection<Class<Serializable>> classes = getSerializableClasses(packageName);
        checkSerialization(classes);
    }

    @SuppressWarnings("unchecked")
    private static Collection<Class<Serializable>> getSerializableClasses(final String packageName) {
        String[] classnames = classpathMonitor.getClasspathClasses();
        Collection<Class<Serializable>> classes = new ArrayList<Class<Serializable>>();
        for (int i = 0; i < classnames.length; i++) {
            if (classnames[i].startsWith(packageName)) {
                try {
                    Class<?> clazz = Class.forName(classnames[i]);
                    if (clazz instanceof Serializable) {
                        classes.add((Class<Serializable>) clazz);
                    }
                } catch (ClassNotFoundException ignored) {
                    log.info(classnames[i] + " will be ignored", ignored);
                }
            }
        }
        return classes;
    }
    
}
